@@ -13,7 +13,7 @@ package MooX::Options;
 use strict;
 use warnings;
 
-our $VERSION = '1.6';    # VERSION
+our $VERSION = '1.7';    # VERSION
 use Carp;
 use Data::Dumper;
 use Getopt::Long 2.38;
@@ -27,10 +27,12 @@ my %DEFAULT_OPTIONS = (
     'option_chain_method'   => 'has',
     'option_method_name'    => 'option',
     'flavour'               => [],
+    'protect_argv'          => 1,
 );
 
 my @FILTER = qw/format short repeatable negativable autosplit doc/;
 
+## no critic qw(ProhibitExcessComplexity)
 sub import {
     my ( undef, %import_params ) = @_;
     my (%import_options) = ( %DEFAULT_OPTIONS, %import_params );
@@ -38,11 +40,12 @@ sub import {
 
     #check options and definition
     while ( my ( $key, $method ) = each %import_options ) {
-        croak "missing option $key, check doc to define one" unless $method;
+        croak "missing option $key, check doc to define one"
+            unless defined $method;
         croak "method $method is not defined, check doc to use another name"
-            if $key =~ /_chain_method$/ && !$caller->can($method);
+            if $key =~ /_chain_method$/x && !$caller->can($method);
         croak "method $method already defined, check doc to use another name"
-            if $key =~ /_method_name$/ && $caller->can($method);
+            if $key =~ /_method_name$/x && $caller->can($method);
     }
 
     my @Options              = ('USAGE: %c %o');
@@ -51,156 +54,163 @@ sub import {
     my %Autosplit_Attributes = ();
     my $Usage                = "";
 
-    {
+    my $sub_ref = {};
 
-        #keyword option
-        no strict 'refs';
-        *{"${caller}::$import_options{option_method_name}"} = sub {
-            my ( $name, %orig_options ) = @_;
-            my %options = %orig_options;
-            croak
-                "Negativable params is not usable with non boolean value, don't pass format to use it !"
-                if $options{negativable} && $options{format};
-            croak "Can't use option with help, it is implied by MooX::Options"
-                if $name eq 'help';
-            croak "Can't use option with "
-                . $import_options{option_method_name}
-                . "_usage, it is implied by MooX::Options"
-                if $name eq $import_options{option_method_name} 
-                    . "_usage";
+    #keywords option
+    $sub_ref->{ $import_options{option_method_name} } = sub {
+        my ( $name, %orig_options ) = @_;
+        my %options = %orig_options;
+        croak
+            "Negativable params is not usable with non boolean value, don't pass format to use it !"
+            if $options{negativable} && $options{format};
+        croak "Can't use option with help, it is implied by MooX::Options"
+            if $name eq 'help';
+        croak "Can't use option with "
+            . $import_options{option_method_name}
+            . "_usage, it is implied by MooX::Options"
+            if $name eq $import_options{option_method_name}
+            . "_usage";
 
-            #fix missing option, autosplit implie repeatable
-            $options{repeatable} = 1 if $options{autosplit};
+        #fix missing option, autosplit implie repeatable
+        $options{repeatable} = 1 if $options{autosplit};
 
-            #help is use for help message only
-            my $name_long_and_short = join "|", grep { defined $_ } $name,
-                $options{short};
+        #help is use for help message only
+        my $name_long_and_short = join "|", grep { defined $_ } $name,
+            $options{short};
 
-            #fix format for negativable or add + if it is a boolean
-            if ( $options{repeatable} ) {
-                if ( $options{format} ) {
-                    $options{format} .= "@"
-                        unless substr( $options{format}, -1 ) eq '@';
-                }
-                else {
-                    $name_long_and_short .= "+";
-                }
+        #fix format for negativable or add + if it is a boolean
+        if ( $options{repeatable} ) {
+            if ( $options{format} ) {
+                $options{format} .= "@"
+                    unless substr( $options{format}, -1 ) eq '@';
             }
+            else {
+                $name_long_and_short .= "+";
+            }
+        }
 
-            #negativable for boolean value
-            $name_long_and_short .= "!" if $options{negativable};
+        #negativable for boolean value
+        $name_long_and_short .= "!" if $options{negativable};
 
-            #format the name
-            my $name_format = join "=",
-                grep { defined $_ } $name_long_and_short, $options{format};
+        #format the name
+        my $name_format = join "=",
+            grep { defined $_ } $name_long_and_short, $options{format};
 
-            #doc
-            my $doc
-                = defined $options{doc} ? $options{doc} : "no doc for $name";
+        #doc
+        my $doc = defined $options{doc} ? $options{doc} : "no doc for $name";
 
-            push @Options, [ $name_format, $doc ]; # prepare option for getopt
-            push @Attributes, $name;    # save the attribute for later use
-            push @Required_Attributes, $name
-                if $options{required};    # save the required attribute
-            $Autosplit_Attributes{$name}
-                = Data::Record->new(
-                { split => $options{autosplit}, unless => $RE{quoted} } )
-                if $options{autosplit};    # save autosplit value
+        push @Options, [ $name_format, $doc ];    # prepare option for getopt
+        push @Attributes,          $name;   # save the attribute for later use
+        push @Required_Attributes, $name
+            if $options{required};          # save the required attribute
+        $Autosplit_Attributes{$name}
+            = Data::Record->new(
+            { split => $options{autosplit}, unless => $RE{quoted} } )
+            if $options{autosplit};         # save autosplit value
 
 #remove bad key for passing to chain_method(has), avoid warnings with Moo/Moose
 #by defaut, keep all key
-            unless ( $import_options{nofilter} ) {
-                delete $orig_options{$_} for @FILTER;
-            }
+        unless ( $import_options{nofilter} ) {
+            delete $orig_options{$_} for @FILTER;
+        }
 
-            #chain to chain_method (has)
-            my $chain_method_name = $import_options{option_chain_method};
-            my $chain_method
-                = eval "package ${caller}; sub {${chain_method_name}(\@_)}";
-            $chain_method->( $name, %orig_options );
-        };
-    }
+        #chain to chain_method (has)
+        my $chain_method_name = $import_options{option_chain_method};
+        ## no critic qw(ProhibitStringyEval)
+        my $chain_method
+            = eval "package ${caller}; sub {${chain_method_name}(\@_)}";
+        ## use critic
+        $chain_method->( $name, %orig_options );
+    };
 
-    {
+    #keyword option_usage
+    $sub_ref->{ $import_options{option_method_name} . "_usage" } = sub {
+        my ( $code, @messages ) = @_;
+        print join( "\n", @messages, $Usage );
+        exit($code);
+    };
 
-        #keyword option
-        no strict 'refs';
-        *{"${caller}::$import_options{option_method_name}_usage"} = sub {
-            my ( $code, @messages ) = @_;
-            print( join( "\n", @messages, $Usage ) ), exit($code);
-            }
-    }
+    #keyword new_with_options
+    $sub_ref->{ $import_options{creation_method_name} } = sub {
+        my ( $self, %params ) = @_;
 
-    {
-
-        #keyword new_with_options
-        no strict 'refs';
-        *{"${caller}::$import_options{creation_method_name}"} = sub {
-            my ( $self, %params ) = @_;
-
-            #ensure all method will be call properly
-            for my $attr (@Attributes) {
-                croak "attribute " 
-                    . $attr
-                    . " isn't defined. You have something wrong in your option_chain_method '"
-                    . $import_options{option_chain_method} . "'."
-                    unless $self->can($attr);
-            }
+        #ensure all method will be call properly
+        for my $attr (@Attributes) {
+            croak "attribute "
+                . $attr
+                . " isn't defined. You have something wrong in your option_chain_method '"
+                . $import_options{option_chain_method} . "'."
+                unless $self->can($attr);
+        }
 
 #if autosplit attributes is present, search and replace in ARGV with full version
 #ex --test=1,2,3 become --test=1 --test=2 --test=3
-            if (%Autosplit_Attributes) {
-                my @new_argv;
+        ## no critic qw(RequireLocalizedPunctuationVars)
+        local @ARGV = @ARGV if $import_options{protect_argv};
+        if (%Autosplit_Attributes) {
+            my @new_argv;
 
-                #parse all argv
-                for my $arg (@ARGV) {
-                    my ( $arg_name, $arg_values ) = split( /=/, $arg, 2 );
-                    $arg_name =~ s/^--?//;
-                    if ( my $rec = $Autosplit_Attributes{$arg_name} ) {
-                        foreach my $record ( $rec->records($arg_values) ) {
+            #parse all argv
+            for my $arg (@ARGV) {
+                my ( $arg_name, $arg_values ) = split( /=/x, $arg, 2 );
+                $arg_name =~ s/^--?//x;
+                if ( my $rec = $Autosplit_Attributes{$arg_name} ) {
+                    foreach my $record ( $rec->records($arg_values) ) {
 
-                            #remove the quoted if exist to chain
-                            $record =~ s/^['"]|['"]$//g;
-                            push @new_argv, "--$arg_name=$record";
-                        }
-                    }
-                    else {
-                        push @new_argv, $arg;
+                        #remove the quoted if exist to chain
+                        $record =~ s/^['"]|['"]$//gx;
+                        push @new_argv, "--$arg_name=$record";
                     }
                 }
-                @ARGV = @new_argv;
+                else {
+                    push @new_argv, $arg;
+                }
             }
+            @ARGV = @new_argv;
+        }
+        ## use critic
 
-            #call describe_options
-            my $usage_method
-                = $self->can("$import_options{option_method_name}_usage");
-            my ( $opt, $usage ) = describe_options(
-                @Options,
-                [ "help|h", "show this help message" ],
-                { getopt_conf => $import_options{flavour} }
-            );
-            $Usage = $usage->text;
-            $usage_method->(0) if $opt->help;
+        #call describe_options
+        my $usage_method
+            = $self->can("$import_options{option_method_name}_usage");
+        my ( $opt, $usage ) = describe_options(
+            @Options,
+            [ "help|h", "show this help message" ],
+            { getopt_conf => $import_options{flavour} }
+        );
+        $Usage = $usage->text;
+        $usage_method->(0) if $opt->help;
 
-            #replace command line attribute in params if params not defined
-            my @existing_attributes = grep {
-                my $attr     = $_;
-                my $attr_val = $opt->$attr;
-                defined $attr_val && !exists $params{$attr}
-            } @Attributes;
-            @params{@existing_attributes} = @$opt{@existing_attributes};
+        #replace command line attribute in params if params not defined
+        my @existing_attributes = grep {
+            my $attr     = $_;
+            my $attr_val = $opt->$attr;
+            defined $attr_val && !exists $params{$attr}
+        } @Attributes;
+        @params{@existing_attributes} = @$opt{@existing_attributes};
 
-            #check required params, if anything missing, display help
-            my @missing_params
-                = grep { !defined $params{$_} } @Required_Attributes;
-            $usage_method->( 1, map {"$_ is missing"} @missing_params )
-                if @missing_params;
+        #check required params, if anything missing, display help
+        my @missing_params
+            = grep { !defined $params{$_} } @Required_Attributes;
+        $usage_method->( 1, map {"$_ is missing"} @missing_params )
+            if @missing_params;
 
-            my $creation_method_name = $import_options{creation_chain_method};
-            my $creation_method      = $caller->can($creation_method_name);
-            $creation_method->( $self, %params );
-        };
+        my $creation_method_name = $import_options{creation_chain_method};
+        my $creation_method      = $caller->can($creation_method_name);
+        $creation_method->( $self, %params );
+    };
+
+    #inject method
+    {
+        ## no critic qw(ProhibitNoStrict)
+        no strict qw/refs/;
+        for my $meth ( keys %$sub_ref ) {
+            *{"${caller}::$meth"} = $sub_ref->{$meth};
+        }
+        ## use critic
     }
+
+    return;
 }
 
 1;
@@ -213,7 +223,7 @@ MooX::Options - add option keywords to your Moo object
 
 =head1 VERSION
 
-version 1.6
+version 1.7
 
 =head1 MooX::Options
 
@@ -268,6 +278,12 @@ want to configure Getopt::Long.
 
 Any flavour is pass to L<Getopt::Long> as a configuration, check the doc to see what is possible.
 
+=item protect_argv
+
+by default, argv is protected. if you want to do something else on it, use this option and it will change the real argv.
+
+    use MooX::Options protect_argv => 0;
+
 =back
 
 =back
@@ -458,6 +474,15 @@ Any bugs or evolution can be submit here :
 
 L<Github|https://github.com/celogeek/MooX-Options>
 
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+https://github.com/celogeek/MooX-Options/issues
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =head1 AUTHOR
 
 celogeek <me@celogeek.com>
@@ -472,3 +497,4 @@ the same terms as the Perl 5 programming language system itself.
 =cut
 
 __END__
+
